package tourplaner.ui;

import javafx.collections.ObservableList;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Orientation;
import javafx.scene.control.*;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.control.cell.PropertyValueFactory;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.web.WebView;
import tourplaner.business.AlertHelper;
import tourplaner.business.LogHelper;
import tourplaner.object.Tour;
import tourplaner.viewmodels.ViewModel;

import java.awt.*;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ResourceBundle;

public class TourplanerController implements Initializable {

    public ViewModel viewModel = new ViewModel();
    public ListView<String> TourListView = new ListView<>();
    public TabPane viewTabPane;
    public Tab kartenTab, beschreibungTab;
    public TableView<Tour> beschreibungTableView;
    public TableColumn<Tour, String> startCol, zielCol, dauerCol, streckeCol, nameCol;
    public TextField titleTextView, sucheInput;

    /**
     * Öffnet github im standart browser
     *
     */
    @FXML
    private void gitWebBrowser(){
       openBrowser("https://git.dergeorg.at/dergeorg/tourplaner", "Git");
    }

    /**
     * Öffnet Java Doc im standart browser
     *
     */
    @FXML
    private void javaDocBrowser(){
       openBrowser("https://git.dergeorg.at/dergeorg/tourplaner", "JavaDoc");
    }

    /**
     * Öffnet Doxygen Doc im standart browser
     *
     */
    @FXML
    private void doxygenDocBrowser(){
        openBrowser("https://git.dergeorg.at/dergeorg/tourplaner", "Doxygen");
    }

    /**
     * Öffnet einen link im Standart Browser
     * @param uriString
     * @param appname
     */
    private void openBrowser(String uriString, String appname){
        Desktop desktop = java.awt.Desktop.getDesktop();
        try {
            URI oURL = new URI(
                    uriString);
            desktop.browse(oURL);
        } catch (URISyntaxException | IOException e) {
            LogHelper.error(e.getMessage(), "TourplanerController -> Browser:" + appname);
            AlertHelper.exerror("Browser Exception", "TourplanerController -> Browser:" + appname, "Error beim öffnen des Browsers", e);
        }
    }

    /**
     * Wird gestartet wenn eine Tour in der Tour listView ausgewählt wird
     * @param mouseEvent Triggered Event
     */
    @FXML
    private void tourListSelectedItem(MouseEvent mouseEvent){
        String selectedItem = TourListView.getSelectionModel().getSelectedItem();
        this.viewModel.selectTour(selectedItem);
        titleTextView.setText(selectedItem);
        beschreibungTableView.getItems().removeIf(s -> true); //Leert die Table View komplett
        beschreibungTableView.getItems().add(this.viewModel.getTour(selectedItem));
        startCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("start"));
        zielCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("ziel"));
        dauerCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("dauer"));
        streckeCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("strecke"));
        nameCol.setCellValueFactory(new PropertyValueFactory<Tour, String>("name"));
    }

    /**
     * Beendet die App
     * Verbunden mit dem Menu -> Datei -> Beenden
     */
    @FXML
    private void quitApp(){
        System.exit(0);
    }

    /**
     * Fügt eine Tour hinzu
     * Verbunden mit Button -> Tour -> +
     */
    @FXML
    private void addTour(){
        this.viewModel.addTour();
    }

    /**
     * Entfernt eine ausgewählte Tour
     * Verbunden mit Button -> Tour -> -
     */
    @FXML
    private void delTour(){
        this.beschreibungTableView.getItems().removeIf(s -> true); //löscht alles aus der tabelle
        this.titleTextView.setText("Keine Tour ausgewählt!");
        this.viewModel.delTour();
    }

    /**
     * Sucht eine Tour
     * Verbunden mit Button -> Suche
     */
    @FXML
    private void suche(){
        String sucheInput = this.sucheInput.getText();
        if(sucheInput.isEmpty()){
            AlertHelper.warn("ACHTUNG", "Suchfeld ist leer!", "Bitte zuerst den Suchtext in das Suchfeld eingeben!");
        }else {
            this.viewModel.suche(sucheInput);
        }
    }

    /**
     * Wird beim Start ausgeführt
     * @param url
     * @param resourceBundle
     */
    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        TourListView.getSelectionModel().setSelectionMode(SelectionMode.SINGLE);
        TourListView.setOrientation(Orientation.VERTICAL);
        TourListView.setItems(this.viewModel.getTourNamen());
        this.beschreibungTableView.setPlaceholder(new Label("Keine Tour ausgewählt!"));
        this.titleTextView.setText("Keine Tour ausgewählt!");
    }
}
